/*
 * Copyright (C) by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file OpenSSLWrapperEVP.h
 *    Wrapper around OpenSSL EVP (envelope encryption) message digest functions
 *
 * @author Christian Martin
 * @date   2023/04/14
 */

#ifndef _MIRA_OPENSSL_WRAPPER_MD_H_
#define _MIRA_OPENSSL_WRAPPER_MD_H_

#include <security/HashStream.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

/**
 * Base class for all message digest hash functions.
 *
 * @ingroup SecurityModule
 */
class MIRA_BASE_EXPORT OpenSSLWrapperMD :
	public HashStreamBuf
{
public:
	/// Default constructor.
	OpenSSLWrapperMD(const std::string& algorithm);

	/// The destructor.
	virtual ~OpenSSLWrapperMD();

public:
	/// Implementation of HashStreamBuf
	virtual void reset();

	/// Implementation of HashStreamBuf
	virtual HashDigest getDigest() const;

protected:
	/// Implementation of std::basic_streambuf
	virtual std::streamsize xsputn(const char* s, std::streamsize n);

private:
	struct Context;

	Context* mCtx;
};

///////////////////////////////////////////////////////////////////////////////

} // namespace

#endif
